@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.33 2008/06/23 16:58:49 nicm Exp $ */
+/* $Id: client.c,v 1.35 2008/09/09 22:16:36 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -34,7 +34,8 @@
 void	client_handle_winch(struct client_ctx *);
 
 int
-client_init(const char *path, struct client_ctx *cctx, int start_server)
+client_init(
+    const char *path, struct client_ctx *cctx, int start_server, int flags)
 {
 	struct sockaddr_un		sa;
 	struct stat			sb;
@@ -93,6 +94,8 @@ retry:
 	if (isatty(STDIN_FILENO)) {
 		if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) == -1)
 			fatal("ioctl(TIOCGWINSZ)");
+		data.version = PROTOCOL_VERSION;
+		data.flags = flags;
 		data.sx = ws.ws_col;
 		data.sy = ws.ws_row;
 		*data.tty = '\0';
