@@ -1,4 +1,4 @@
-/* $Id: cmd-command-prompt.c,v 1.4 2008/06/25 20:43:13 nicm Exp $ */
+/* $Id: cmd-command-prompt.c,v 1.6 2008/09/26 06:45:25 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -72,7 +72,7 @@ cmd_command_prompt_callback(void *data, char *s)
 	if (s == NULL)
 		return;
 
-	if ((cmd = cmd_string_parse(s, &cause)) == NULL) {
+	if (cmd_string_parse(s, &cmd, &cause) != 0) {
 		if (cause == NULL)
 			return;
 		*cause = toupper((u_char) *cause);
@@ -80,7 +80,9 @@ cmd_command_prompt_callback(void *data, char *s)
 		xfree(cause);
 		return;
 	}
-	
+	if (cmd == NULL)
+		return;
+
 	ctx.msgdata = NULL;
 	ctx.cursession = c->session;
 	ctx.curclient = c;
