@@ -1,4 +1,4 @@
-/* $Id: cmd-new-session.c,v 1.30 2008/06/21 10:19:36 nicm Exp $ */
+/* $Id: cmd-new-session.c,v 1.31 2008/09/25 23:28:12 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -75,7 +75,7 @@ cmd_new_session_parse(struct cmd *self, int argc, char **argv, char **cause)
 	self->entry->init(self, 0);
 	data = self->data;
 
-	while ((opt = getopt(argc, argv, "ds:n:")) != EOF) {
+	while ((opt = getopt(argc, argv, GETOPT_PREFIX "ds:n:")) != EOF) {
 		switch (opt) {
 		case 'd':
 			data->flag_detached = 1;
