@@ -1,4 +1,4 @@
-/* $Id: cmd-set-option.c,v 1.38 2008/07/01 19:00:50 nicm Exp $ */
+/* $Id: cmd-set-option.c,v 1.44 2008/11/16 13:28:59 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -55,14 +55,19 @@ const struct cmd_entry cmd_set_option_entry = {
 	cmd_set_option_print
 };
 
-const char *set_option_bell_action_choices[] = { "none", "any", "current", NULL };
+const char *set_option_bell_action_list[] = {
+	"none", "any", "current", NULL
+};
+const char *set_option_mode_keys_list[] = {
+	"emacs", "vi", NULL
+};
 const struct set_option_entry set_option_table[NSETOPTION] = {
-	{ "bell-action",
-	  SET_OPTION_CHOICE, 0, 0, set_option_bell_action_choices },
+	{ "bell-action", SET_OPTION_CHOICE, 0, 0, set_option_bell_action_list },
 	{ "buffer-limit", SET_OPTION_NUMBER, 1, INT_MAX, NULL },
 	{ "default-command", SET_OPTION_STRING, 0, 0, NULL },
 	{ "display-time", SET_OPTION_NUMBER, 1, INT_MAX, NULL },
 	{ "history-limit", SET_OPTION_NUMBER, 0, SHRT_MAX, NULL },
+	{ "mode-keys", SET_OPTION_CHOICE, 0, 0, set_option_mode_keys_list },
 	{ "prefix", SET_OPTION_KEY, 0, 0, NULL },
 	{ "remain-by-default", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "set-titles", SET_OPTION_FLAG, 0, 0, NULL },
@@ -72,9 +77,10 @@ const struct set_option_entry set_option_table[NSETOPTION] = {
 	{ "status-interval", SET_OPTION_NUMBER, 0, INT_MAX, NULL },
 	{ "status-left", SET_OPTION_STRING, 0, 0, NULL },
 	{ "status-right", SET_OPTION_STRING, 0, 0, NULL },
+	{ "utf8", SET_OPTION_FLAG, 0, 0, NULL },
 };
 
-void	set_option_string(struct cmd_ctx *, 
+void	set_option_string(struct cmd_ctx *,
 	    struct options *, const struct set_option_entry *, char *);
 void	set_option_number(struct cmd_ctx *,
     	    struct options *, const struct set_option_entry *, char *);
@@ -99,7 +105,7 @@ cmd_set_option_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data->option = NULL;
 	data->value = NULL;
 
-	while ((opt = getopt(argc, argv, "t:s:")) != EOF) {
+	while ((opt = getopt(argc, argv, GETOPT_PREFIX "t:s:")) != EOF) {
 		switch (opt) {
 		case 't':
 			if (data->target == NULL)
@@ -163,6 +169,9 @@ cmd_set_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 		entry = &set_option_table[i];
 
+		/* Bail now if an exact match. */
+		if (strcmp(entry->name, data->option) == 0)
+			break;
 	}
 	if (entry == NULL) {
 		ctx->error(ctx, "unknown option: %s", data->option);
@@ -232,7 +241,7 @@ set_option_number(struct cmd_ctx *ctx, struct options *oo,
 	}
 	options_set_number(oo, entry->name, number);
 }
-	
+
 void
 set_option_key(struct cmd_ctx *ctx, struct options *oo,
     const struct set_option_entry *entry, char *value)
@@ -249,9 +258,9 @@ set_option_key(struct cmd_ctx *ctx, struct options *oo,
 		return;
 	}
 	options_set_number(oo, entry->name, key);
-	
+
 }
-	
+
 void
 set_option_colour(struct cmd_ctx *ctx, struct options *oo,
     const struct set_option_entry *entry, char *value)
@@ -262,21 +271,21 @@ set_option_colour(struct cmd_ctx *ctx, struct options *oo,
 		ctx->error(ctx, "empty value");
 		return;
 	}
-	
-	if ((colour = screen_stringcolour(value)) > 8) {
+
+	if ((colour = colour_fromstring(value)) > 8) {
 		ctx->error(ctx, "bad colour: %s", value);
 		return;
 	}
-	
-	options_set_number(oo, entry->name, colour);	
+
+	options_set_number(oo, entry->name, colour);
 }
-	
+
 void
 set_option_flag(struct cmd_ctx *ctx, struct options *oo,
     const struct set_option_entry *entry, char *value)
 {
 	int	flag;
-	
+
 	if (value == NULL || *value == '\0')
 		flag = !options_get_number(oo, entry->name);
 	else {
@@ -296,7 +305,7 @@ set_option_flag(struct cmd_ctx *ctx, struct options *oo,
 
 	options_set_number(oo, entry->name, flag);
 }
-	
+
 void
 set_option_choice(struct cmd_ctx *ctx, struct options *oo,
     const struct set_option_entry *entry, char *value)
