@@ -1,4 +1,4 @@
-/* $Id: cmd-show-options.c,v 1.5 2008/06/23 22:12:29 nicm Exp $ */
+/* $Id: cmd-show-options.c,v 1.7 2008/09/25 23:28:15 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -67,7 +67,7 @@ cmd_show_options_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data->target = NULL;
 	data->flag_global = 1;
 
-	while ((opt = getopt(argc, argv, "t:s:")) != EOF) {
+	while ((opt = getopt(argc, argv, GETOPT_PREFIX "t:s:")) != EOF) {
 		switch (opt) {
 		case 't':
 			if (data->target == NULL)
@@ -135,7 +135,7 @@ cmd_show_options_exec(struct cmd *self, struct cmd_ctx *ctx)
 		case SET_OPTION_COLOUR:
 			vn = options_get_number(oo, entry->name);
  			ctx->print(ctx, "%s %s",
-			    entry->name, screen_colourstring(vn));
+			    entry->name, colour_tostring(vn));
 			break;
 		case SET_OPTION_FLAG:
 			vn = options_get_number(oo, entry->name);
