@@ -1,4 +1,4 @@
-/* $Id: cmd.c,v 1.60 2008/06/29 07:04:30 nicm Exp $ */
+/* $Id: cmd.c,v 1.65 2008/09/26 06:45:26 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -93,13 +93,13 @@ cmd_complete(const char *s)
 			continue;
 		ARRAY_ADD(&list, (*entryp)->name);
 	}
-	
+
 	/* If none, bail now with the original string. */
 	if (ARRAY_LENGTH(&list) == 0) {
 		ARRAY_FREE(&list);
 		return (xstrdup(s));
 	}
-	
+
 	/* If an exact match, return it, with a trailing space. */
 	if (ARRAY_LENGTH(&list) == 1) {
 		xasprintf(&s2, "%s ", ARRAY_FIRST(&list));
@@ -150,6 +150,10 @@ cmd_parse(int argc, char **argv, char **cause)
 		if (entry != NULL)
 			goto ambiguous;
 		entry = *entryp;
+
+		/* Bail now if an exact match. */
+		if (strcmp(entry->name, argv[0]) == 0)
+			break;
 	}
 	if (entry == NULL) {
 		xasprintf(cause, "unknown command: %s", argv[0]);
@@ -158,7 +162,7 @@ cmd_parse(int argc, char **argv, char **cause)
 
 	optind = 1;
 	if (entry->parse == NULL) {
-		while ((opt = getopt(argc, argv, "")) != EOF) {
+		while ((opt = getopt(argc, argv, GETOPT_PREFIX "")) != EOF) {
 			switch (opt) {
 			default:
 				goto usage;
@@ -300,7 +304,7 @@ struct session *
 cmd_current_session(struct cmd_ctx *ctx)
 {
 	struct msg_command_data	*data = ctx->msgdata;
-	struct timespec		*ts;
+	struct timeval		*tv;
 	struct session		*s, *newest = NULL;
 	u_int			 i;
 
@@ -309,7 +313,7 @@ cmd_current_session(struct cmd_ctx *ctx)
 
 	if (data != NULL && data->pid != -1) {
 		if (data->pid != getpid()) {
-			ctx->error(ctx, "wrong server: %lld", data->pid);
+			ctx->error(ctx, "wrong server: %ld", (long) data->pid);
 			return (NULL);
 		}
 		if (data->idx > ARRAY_LENGTH(&sessions)) {
@@ -323,12 +327,12 @@ cmd_current_session(struct cmd_ctx *ctx)
 		return (s);
 	}
 
-	ts = NULL;
+	tv = NULL;
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
-		if (s != NULL && (ts == NULL || timespeccmp(&s->ts, ts, >))) {
+		if (s != NULL && (tv == NULL || timercmp(&s->tv, tv, >))) {
 			newest = ARRAY_ITEM(&sessions, i);
-			ts = &s->ts;
+			tv = &s->tv;
 		}
 	}
 	return (newest);
