@@ -1,4 +1,4 @@
-/* $Id: input.c,v 1.50 2008/06/18 19:06:51 nicm Exp $ */
+/* $Id: input.c,v 1.67 2008/11/16 13:28:59 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -48,14 +48,12 @@ void	 input_state(struct input_ctx *, void *);
 void	 input_state_first(u_char, struct input_ctx *);
 void	 input_state_escape(u_char, struct input_ctx *);
 void	 input_state_intermediate(u_char, struct input_ctx *);
-void	 input_state_title_first(u_char, struct input_ctx *);
-void	 input_state_title_second(u_char, struct input_ctx *);
-void	 input_state_title_next(u_char, struct input_ctx *);
 void	 input_state_sequence_first(u_char, struct input_ctx *);
 void	 input_state_sequence_next(u_char, struct input_ctx *);
 void	 input_state_sequence_intermediate(u_char, struct input_ctx *);
 void	 input_state_string_next(u_char, struct input_ctx *);
 void	 input_state_string_escape(u_char, struct input_ctx *);
+void	 input_state_utf8(u_char, struct input_ctx *);
 
 void	 input_handle_character(u_char, struct input_ctx *);
 void	 input_handle_c0_control(u_char, struct input_ctx *);
@@ -185,12 +183,20 @@ input_state(struct input_ctx *ictx, void *state)
 void
 input_init(struct window *w)
 {
-	ARRAY_INIT(&w->ictx.args);
+	struct input_ctx	*ictx = &w->ictx;
 
-	w->ictx.string_len = 0;
-	w->ictx.string_buf = NULL;
+	ARRAY_INIT(&ictx->args);
 
-	input_state(&w->ictx, input_state_first);
+	ictx->string_len = 0;
+	ictx->string_buf = NULL;
+
+ 	memcpy(&ictx->cell, &grid_default_cell, sizeof ictx->cell);
+
+	memcpy(&ictx->saved_cell, &grid_default_cell, sizeof ictx->saved_cell);
+	ictx->saved_cx = 0;
+	ictx->saved_cy = 0;
+
+	input_state(ictx, input_state_first);
 }
 
 void
@@ -239,6 +245,8 @@ input_parse(struct window *w)
 void
 input_state_first(u_char ch, struct input_ctx *ictx)
 {
+	ictx->intermediate = '\0';
+
 	if (INPUT_C0CONTROL(ch)) {
 		if (ch == 0x1b)
 			input_state(ictx, input_state_escape);
@@ -247,16 +255,25 @@ input_state_first(u_char ch, struct input_ctx *ictx)
 		return;
 	}
 
+#if 0
   	if (INPUT_C1CONTROL(ch)) {
 		ch -= 0x40;
 		if (ch == '[')
 			input_state(ictx, input_state_sequence_first);
-		else if (ch == ']')
-			input_state(ictx, input_state_title_first);
-		else
+		else if (ch == ']') {
+			input_start_string(ictx, STRING_SYSTEM);
+			input_state(ictx, input_state_string_next);
+		} else if (ch == '_') {
+			input_start_string(ictx, STRING_APPLICATION);
+			input_state(ictx, input_state_string_next);
+		} else
 			input_handle_c1_control(ch, ictx);
 		return;
 	}
+#endif
+
+	if (INPUT_DELETE(ch))
+		return;
 
 	input_handle_character(ch, ictx);
 }
@@ -274,6 +291,8 @@ input_state_escape(u_char ch, struct input_ctx *ictx)
 	}
 
 	if (INPUT_INTERMEDIATE(ch)) {
+		log_debug2(":: in1 %zu: %hhu (%c)", ictx->off, ch, ch);
+		ictx->intermediate = ch;
 		input_state(ictx, input_state_intermediate);
 		return;
 	}
@@ -287,9 +306,13 @@ input_state_escape(u_char ch, struct input_ctx *ictx)
 	if (INPUT_UPPERCASE(ch)) {
 		if (ch == '[')
 			input_state(ictx, input_state_sequence_first);
-		else if (ch == ']')
-			input_state(ictx, input_state_title_first);
-		else {
+		else if (ch == ']') {
+			input_start_string(ictx, STRING_SYSTEM);
+			input_state(ictx, input_state_string_next);
+		} else if (ch == '_') {
+			input_start_string(ictx, STRING_APPLICATION);
+			input_state(ictx, input_state_string_next);
+		} else {
 			input_state(ictx, input_state_first);
 			input_handle_c1_control(ch, ictx);
 		}
@@ -305,59 +328,14 @@ input_state_escape(u_char ch, struct input_ctx *ictx)
 	input_state(ictx, input_state_first);
 }
 
-void
-input_state_title_first(u_char ch, struct input_ctx *ictx)
-{
-	if (ch >= '0' && ch <= '9') {
-		if (ch == '0')
-			input_start_string(ictx, STRING_TITLE);
-		else
-			input_start_string(ictx, STRING_IGNORE);
-		input_state(ictx, input_state_title_second);
-		return;
-	}
-
-	input_state(ictx, input_state_first);
-}
-
-void
-input_state_title_second(u_char ch, struct input_ctx *ictx)
-{
-	if (ch == ';') {
-		input_state(ictx, input_state_title_next);
-		return;
-	}
-
-	input_state(ictx, input_state_first);
-}
-
-void
-input_state_title_next(u_char ch, struct input_ctx *ictx)
-{
-	if (ch == '\007') {
-		if (ictx->string_type == STRING_TITLE) {
-			screen_write_set_title(
-			    &ictx->ctx, input_get_string(ictx));
-		} else
-			input_abort_string(ictx);
-		input_state(ictx, input_state_first);
-		return;
-	}
-
-	if (ch >= 0x20 && ch != 0x7f) {
-		if (input_add_string(ictx, ch) != 0)
-			input_state(ictx, input_state_first);
-		return;
-	}
-
-	input_state(ictx, input_state_first);
-}
-
 void
 input_state_intermediate(u_char ch, struct input_ctx *ictx)
 {
-	if (INPUT_INTERMEDIATE(ch))
+	if (INPUT_INTERMEDIATE(ch)) {
+		/* Multiple intermediates currently ignored. */
+		log_debug2(":: in2 %zu: %hhu (%c)", ictx->off, ch, ch);
 		return;
+	}
 
 	if (INPUT_PARAMETER(ch)) {
 		input_state(ictx, input_state_first);
@@ -401,8 +379,10 @@ input_state_sequence_next(u_char ch, struct input_ctx *ictx)
 	if (INPUT_INTERMEDIATE(ch)) {
 		if (input_add_argument(ictx, '\0') != 0)
 			input_state(ictx, input_state_first);
-		else
+		else {
+			log_debug2(":: si1 %zu: %hhu (%c)", ictx->off, ch, ch);
 			input_state(ictx, input_state_sequence_intermediate);
+		}
 		return;
 	}
 
@@ -433,8 +413,10 @@ input_state_sequence_next(u_char ch, struct input_ctx *ictx)
 void
 input_state_sequence_intermediate(u_char ch, struct input_ctx *ictx)
 {
-	if (INPUT_INTERMEDIATE(ch))
+	if (INPUT_INTERMEDIATE(ch)) {
+		log_debug2(":: si2 %zu: %hhu (%c)", ictx->off, ch, ch);
 		return;
+	}
 
 	if (INPUT_UPPERCASE(ch) || INPUT_LOWERCASE(ch)) {
 		input_state(ictx, input_state_first);
@@ -452,6 +434,10 @@ input_state_string_next(u_char ch, struct input_ctx *ictx)
 		input_state(ictx, input_state_string_escape);
 		return;
 	}
+	if (ch == 0x07) {
+		input_state_string_escape(ch, ictx);
+		return;
+	}
 
 	if (ch >= 0x20 && ch != 0x7f) {
 		if (input_add_string(ictx, ch) != 0)
@@ -463,10 +449,32 @@ input_state_string_next(u_char ch, struct input_ctx *ictx)
 void
 input_state_string_escape(u_char ch, struct input_ctx *ictx)
 {
-	if (ch == '\\') {
+	char	*s;
+
+	if (ch == '\007' || ch == '\\') {
 		input_state(ictx, input_state_first);
 		switch (ictx->string_type) {
+		case STRING_SYSTEM:
+			if (ch != '\007')
+				return;
+			s = input_get_string(ictx);
+			if ((s[0] != '0' && s[0] != '2') || s[1] != ';') {
+				xfree(s);
+				return;
+			}
+			screen_set_title(ictx->ctx.s, s + 2);
+			xfree(s);
+			break;
+		case STRING_APPLICATION:
+			if (ch != '\\')
+				return;
+			s = input_get_string(ictx);
+			screen_set_title(ictx->ctx.s, s);
+			xfree(s);
+			break;
 		case STRING_NAME:
+			if (ch != '\\')
+				return;
 			xfree(ictx->w->name);
 			ictx->w->name = input_get_string(ictx);
 			server_status_window(ictx->w);
@@ -479,19 +487,70 @@ input_state_string_escape(u_char ch, struct input_ctx *ictx)
 	input_state_string_next(ch, ictx);
 }
 
+void
+input_state_utf8(u_char ch, struct input_ctx *ictx)
+{
+	u_int	value;
+
+	log_debug2("-- un %zu: %hhu (%c)", ictx->off, ch, ch);
+
+	ictx->utf8_buf[ictx->utf8_off++] = ch;
+	if (--ictx->utf8_len != 0)
+		return;
+	input_state(ictx, input_state_first);
+
+	value = utf8_combine(ictx->utf8_buf);
+	if (value > 0xffff)	/* non-BMP not supported */
+		value = '_';
+
+	ictx->cell.data = value;
+	screen_write_cell(&ictx->ctx, &ictx->cell);
+}
+
 void
 input_handle_character(u_char ch, struct input_ctx *ictx)
 {
+	if (ictx->w->flags & WINDOW_UTF8 && ch > 0x7f) {
+		/*
+		 * UTF-8 sequence.
+		 *
+		 * 11000010-11011111 C2-DF start of 2-byte sequence
+		 * 11100000-11101111 E0-EF start of 3-byte sequence
+		 * 11110000-11110100 F0-F4 start of 4-byte sequence
+		 */
+		memset(ictx->utf8_buf, 0xff, sizeof ictx->utf8_buf);
+		ictx->utf8_buf[0] = ch;
+		ictx->utf8_off = 1;
+
+		if (ch >= 0xc2 && ch <= 0xdf) {
+			log_debug2("-- u2 %zu: %hhu (%c)", ictx->off, ch, ch);
+			input_state(ictx, input_state_utf8);
+			ictx->utf8_len = 1;
+			return;
+		}
+		if (ch >= 0xe0 && ch <= 0xef) {
+			log_debug2("-- u3 %zu: %hhu (%c)", ictx->off, ch, ch);
+			input_state(ictx, input_state_utf8);
+			ictx->utf8_len = 2;
+			return;
+		}
+		if (ch >= 0xf0 && ch <= 0xf4) {
+			log_debug2("-- u4 %zu: %hhu (%c)", ictx->off, ch, ch);
+			input_state(ictx, input_state_utf8);
+			ictx->utf8_len = 3;
+			return;
+		}
+	}
 	log_debug2("-- ch %zu: %hhu (%c)", ictx->off, ch, ch);
 
-	screen_write_put_character(&ictx->ctx, ch);
+	ictx->cell.data = ch;
+	screen_write_cell(&ictx->ctx, &ictx->cell);
 }
 
 void
 input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 {
 	struct screen	*s = ictx->ctx.s;
-	u_char		 attr;
 
 	log_debug2("-- c0 %zu: %hhu", ictx->off, ch);
 
@@ -499,32 +558,31 @@ input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 	case '\0':	/* NUL */
 		break;
 	case '\n':	/* LF */
-		screen_write_cursor_down_scroll(&ictx->ctx);
+		screen_write_linefeed(&ictx->ctx);
 		break;
 	case '\r':	/* CR */
-		screen_write_move_cursor(&ictx->ctx, 0, s->cy);
+		screen_write_carriagereturn(&ictx->ctx);
 		break;
 	case '\007':	/* BELL */
 		ictx->w->flags |= WINDOW_BELL;
 		break;
 	case '\010': 	/* BS */
-		screen_write_cursor_left(&ictx->ctx, 1);
+		screen_write_cursorleft(&ictx->ctx, 1);
 		break;
 	case '\011': 	/* TAB */
+		/* XXX right? */
 		s->cx = ((s->cx / 8) * 8) + 8;
-		if (s->cx > screen_last_x(s)) {
+		if (s->cx > screen_size_x(s) - 1) {
 			s->cx = 0;
-			screen_write_cursor_down(&ictx->ctx, 1);
+			screen_write_cursordown(&ictx->ctx, 1);
 		}
-		screen_write_move_cursor(&ictx->ctx, s->cx, s->cy);
+		screen_write_cursormove(&ictx->ctx, s->cx, s->cy);
 		break;
 	case '\016':	/* SO */
-		attr = s->attr | ATTR_CHARSET;
-		screen_write_set_attributes(&ictx->ctx, attr, s->colr);
+		ictx->cell.attr |= GRID_ATTR_CHARSET;
 		break;
 	case '\017':	/* SI */
-		attr = s->attr & ~ATTR_CHARSET;
-		screen_write_set_attributes(&ictx->ctx, attr, s->colr);
+		ictx->cell.attr &= ~GRID_ATTR_CHARSET;
 		break;
 	default:
 		log_debug("unknown c0: %hhu", ch);
@@ -539,7 +597,7 @@ input_handle_c1_control(u_char ch, struct input_ctx *ictx)
 
 	switch (ch) {
 	case 'M':	/* RI */
-		screen_write_cursor_up_scroll(&ictx->ctx);
+		screen_write_reverseindex(&ictx->ctx);
 		break;
 	default:
 		log_debug("unknown c1: %hhu", ch);
@@ -552,30 +610,39 @@ input_handle_private_two(u_char ch, struct input_ctx *ictx)
 {
 	struct screen	*s = ictx->ctx.s;
 
-	log_debug2("-- p2 %zu: %hhu (%c)", ictx->off, ch, ch);
+	log_debug2(
+	    "-- p2 %zu: %hhu (%c) %hhu", ictx->off, ch, ch, ictx->intermediate);
 
 	switch (ch) {
-	case '=':	/* DECKPAM */
-		screen_write_set_mode(&ictx->ctx, MODE_KKEYPAD);
-		log_debug("kkeypad on");
+	case '0':	/* Dscs (graphics) */
+		/* 
+		 * Not really supported, but fake it up enough for those that
+		 * use it to switch character sets (by redefining G0 to
+		 * graphics set, rather than switching to G1).
+		 */
+		switch (ictx->intermediate) {
+		case '(':	/* G0 */
+			ictx->cell.attr |= GRID_ATTR_CHARSET;
+			break;
+		}
 		break;
-	case '>':	/* DECKPNM*/
-		screen_write_clear_mode(&ictx->ctx, MODE_KKEYPAD);
-		log_debug("kkeypad off");
+	case '=':	/* DECKPAM */
+		screen_write_kkeypadmode(&ictx->ctx, 1);
+		log_debug("kkeypad on (application mode)");
+		break;
+	case '>':	/* DECKPNM */
+		screen_write_kkeypadmode(&ictx->ctx, 0);
+		log_debug("kkeypad off (number mode)");
 		break;
 	case '7':	/* DECSC */
-		s->saved_cx = s->cx;
-		s->saved_cy = s->cy;
-		s->saved_attr = s->attr;
-		s->saved_colr = s->colr;
-		s->mode |= MODE_SAVED;
+		memcpy(&ictx->saved_cell, &ictx->cell, sizeof ictx->saved_cell);
+		ictx->saved_cx = s->cx;
+		ictx->saved_cy = s->cy;
 		break;
 	case '8':	/* DECRC */
-		if (!(s->mode & MODE_SAVED))
-			break;
-		screen_write_set_attributes(
-		    &ictx->ctx, s->saved_attr, s->saved_colr);
-		screen_write_move_cursor(&ictx->ctx, s->saved_cx, s->saved_cy);
+		memcpy(&ictx->cell, &ictx->saved_cell, sizeof ictx->cell);
+		screen_write_cursormove(
+		    &ictx->ctx, ictx->saved_cx, ictx->saved_cy);
 		break;
 	default:
 		log_debug("unknown p2: %hhu", ch);
@@ -586,9 +653,40 @@ input_handle_private_two(u_char ch, struct input_ctx *ictx)
 void
 input_handle_standard_two(u_char ch, struct input_ctx *ictx)
 {
-	log_debug2("-- s2 %zu: %hhu (%c)", ictx->off, ch, ch);
+	log_debug2(
+	    "-- s2 %zu: %hhu (%c) %hhu", ictx->off, ch, ch, ictx->intermediate);
 
 	switch (ch) {
+	case 'B':	/* Dscs (ASCII) */
+		/* 
+		 * Not really supported, but fake it up enough for those that
+		 * use it to switch character sets (by redefining G0 to
+		 * graphics set, rather than switching to G1).
+		 */
+		switch (ictx->intermediate) {
+		case '(':	/* G0 */
+			ictx->cell.attr &= ~GRID_ATTR_CHARSET;
+			break;
+		}
+		break;
+	case 'c':	/* RIS */
+		memcpy(&ictx->cell, &grid_default_cell, sizeof ictx->cell);
+
+		memcpy(&ictx->saved_cell, &ictx->cell, sizeof ictx->saved_cell);
+		ictx->saved_cx = 0;
+		ictx->saved_cy = 0;
+
+		screen_write_scrollregion(
+		    &ictx->ctx, 0, screen_size_y(ictx->ctx.s) - 1);
+
+		screen_write_insertmode(&ictx->ctx, 0);
+		screen_write_kcursormode(&ictx->ctx, 0);
+		screen_write_kkeypadmode(&ictx->ctx, 0);
+		screen_write_mousemode(&ictx->ctx, 0);
+
+		screen_write_clearscreen(&ictx->ctx);
+		screen_write_cursormove(&ictx->ctx, 0, 0);
+		break;
 	case 'k':
 		input_start_string(ictx, STRING_NAME);
 		input_state(ictx, input_state_string_next);
@@ -666,7 +764,7 @@ input_handle_sequence_cuu(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_cursor_up(&ictx->ctx, n);
+	screen_write_cursorup(&ictx->ctx, n);
 }
 
 void
@@ -684,7 +782,7 @@ input_handle_sequence_cud(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_cursor_down(&ictx->ctx, n);
+	screen_write_cursordown(&ictx->ctx, n);
 }
 
 void
@@ -702,7 +800,7 @@ input_handle_sequence_cuf(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_cursor_right(&ictx->ctx, n);
+	screen_write_cursorright(&ictx->ctx, n);
 }
 
 void
@@ -720,7 +818,7 @@ input_handle_sequence_cub(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_cursor_left(&ictx->ctx, n);
+	screen_write_cursorleft(&ictx->ctx, n);
 }
 
 void
@@ -738,7 +836,7 @@ input_handle_sequence_dch(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_delete_characters(&ictx->ctx, n);
+	screen_write_deletecharacter(&ictx->ctx, n);
 }
 
 void
@@ -756,7 +854,7 @@ input_handle_sequence_dl(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_delete_lines(&ictx->ctx, n);
+	screen_write_deleteline(&ictx->ctx, n);
 }
 
 void
@@ -774,7 +872,7 @@ input_handle_sequence_ich(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_insert_characters(&ictx->ctx, n);
+	screen_write_insertcharacter(&ictx->ctx, n);
 }
 
 void
@@ -792,7 +890,7 @@ input_handle_sequence_il(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_insert_lines(&ictx->ctx, n);
+	screen_write_insertline(&ictx->ctx, n);
 }
 
 void
@@ -811,7 +909,7 @@ input_handle_sequence_vpa(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_move_cursor(&ictx->ctx, s->cx, n - 1);
+	screen_write_cursormove(&ictx->ctx, s->cx, n - 1);
 }
 
 void
@@ -830,7 +928,7 @@ input_handle_sequence_hpa(struct input_ctx *ictx)
 	if (n == 0)
 		n = 1;
 
-	screen_write_move_cursor(&ictx->ctx, n - 1, s->cy);
+	screen_write_cursormove(&ictx->ctx, n - 1, s->cy);
 }
 
 void
@@ -852,7 +950,7 @@ input_handle_sequence_cup(struct input_ctx *ictx)
 	if (m == 0)
 		m = 1;
 
-	screen_write_move_cursor(&ictx->ctx, m - 1, n - 1);
+	screen_write_cursormove(&ictx->ctx, m - 1, n - 1);
 }
 
 void
@@ -872,10 +970,13 @@ input_handle_sequence_ed(struct input_ctx *ictx)
 
 	switch (n) {
 	case 0:
-		screen_write_fill_end_of_screen(&ictx->ctx);
+		screen_write_clearendofscreen(&ictx->ctx);
+		break;
+	case 1:
+		screen_write_clearstartofscreen(&ictx->ctx);
 		break;
 	case 2:
-		screen_write_fill_screen(&ictx->ctx);
+		screen_write_clearscreen(&ictx->ctx);
 		break;
 	}
 }
@@ -897,13 +998,13 @@ input_handle_sequence_el(struct input_ctx *ictx)
 
 	switch (n) {
 	case 0:
-		screen_write_fill_end_of_line(&ictx->ctx);
+		screen_write_clearendofline(&ictx->ctx);
 		break;
 	case 1:
-		screen_write_fill_start_of_line(&ictx->ctx);
+		screen_write_clearstartofline(&ictx->ctx);
 		break;
 	case 2:
-		screen_write_fill_line(&ictx->ctx);
+		screen_write_clearline(&ictx->ctx);
 		break;
 	}
 }
@@ -921,15 +1022,15 @@ input_handle_sequence_sm(struct input_ctx *ictx)
 	if (ictx->private == '?') {
 		switch (n) {
 		case 1:		/* GATM */
-			screen_write_set_mode(&ictx->ctx, MODE_KCURSOR);
+			screen_write_kcursormode(&ictx->ctx, 1);
 			log_debug("kcursor on");
 			break;
 		case 25:	/* TCEM */
-			screen_write_set_mode(&ictx->ctx, MODE_CURSOR);
+			screen_write_cursormode(&ictx->ctx, 1);
 			log_debug("cursor on");
 			break;
 		case 1000:
-			screen_write_set_mode(&ictx->ctx, MODE_MOUSE);
+			screen_write_mousemode(&ictx->ctx, 1);
 			log_debug("mouse on");
 			break;
 		default:
@@ -939,7 +1040,7 @@ input_handle_sequence_sm(struct input_ctx *ictx)
 	} else {
 		switch (n) {
 		case 4:		/* IRM */
-			screen_write_set_mode(&ictx->ctx, MODE_INSERT);
+			screen_write_insertmode(&ictx->ctx, 1);
 			log_debug("insert on");
 			break;
 		case 34:
@@ -965,15 +1066,15 @@ input_handle_sequence_rm(struct input_ctx *ictx)
 	if (ictx->private == '?') {
 		switch (n) {
 		case 1:		/* GATM */
-			screen_write_clear_mode(&ictx->ctx, MODE_KCURSOR);
+			screen_write_kcursormode(&ictx->ctx, 0);
 			log_debug("kcursor off");
 			break;
 		case 25:	/* TCEM */
-			screen_write_clear_mode(&ictx->ctx, MODE_CURSOR);
+			screen_write_cursormode(&ictx->ctx, 0);
 			log_debug("cursor off");
 			break;
 		case 1000:
-			screen_write_clear_mode(&ictx->ctx, MODE_MOUSE);
+			screen_write_mousemode(&ictx->ctx, 0);
 			log_debug("mouse off");
 			break;
 		default:
@@ -983,7 +1084,7 @@ input_handle_sequence_rm(struct input_ctx *ictx)
 	} else if (ictx->private == '\0') {
 		switch (n) {
 		case 4:		/* IRM */
-			screen_write_clear_mode(&ictx->ctx, MODE_INSERT);
+			screen_write_insertmode(&ictx->ctx, 0);
 			log_debug("insert off");
 			break;
 		case 34:
@@ -1041,92 +1142,121 @@ input_handle_sequence_decstbm(struct input_ctx *ictx)
 	if (m == 0)
 		m = screen_size_y(s);
 
-	screen_write_set_region(&ictx->ctx, n - 1, m - 1);
+	screen_write_scrollregion(&ictx->ctx, n - 1, m - 1);
 }
 
 void
 input_handle_sequence_sgr(struct input_ctx *ictx)
 {
-	struct screen  *s = ictx->ctx.s;
-	u_int		i, n;
-	uint16_t	m;
-	u_char		attr, colr;
+	struct grid_cell       *gc = &ictx->cell;
+	u_int			i;
+	uint16_t		m, o;
+	u_char			attr;
 
-	n = ARRAY_LENGTH(&ictx->args);
-	if (n == 0) {
-		attr = 0;
-		colr = SCREEN_DEFCOLR;
-	} else {
-		attr = s->attr;
-		colr = s->colr;
-		for (i = 0; i < n; i++) {
-			if (input_get_argument(ictx, i, &m, 0) != 0)
+	if (ARRAY_LENGTH(&ictx->args) == 0) {
+		attr = gc->attr;
+		memcpy(gc, &grid_default_cell, sizeof *gc);
+ 		gc->attr |= (attr & GRID_ATTR_CHARSET);
+		return;
+	}
+
+	for (i = 0; i < ARRAY_LENGTH(&ictx->args); i++) {
+		if (input_get_argument(ictx, i, &m, 0) != 0)
+			return;
+
+		if (m == 38 || m == 48) {
+			i++;
+			if (input_get_argument(ictx, i, &o, 0) != 0)
 				return;
-			switch (m) {
-			case 0:
-			case 10:
-				attr &= ATTR_CHARSET;
-				colr = SCREEN_DEFCOLR;
-				break;
-			case 1:
-				attr |= ATTR_BRIGHT;
-				break;
-			case 2:
-				attr |= ATTR_DIM;
-				break;
-			case 3:
-				attr |= ATTR_ITALICS;
-				break;
-			case 4:
-				attr |= ATTR_UNDERSCORE;
-				break;
-			case 5:
-				attr |= ATTR_BLINK;
-				break;
-			case 7:
-				attr |= ATTR_REVERSE;
-				break;
-			case 8:
-				attr |= ATTR_HIDDEN;
-				break;
-			case 23:
-				attr &= ~ATTR_ITALICS;
-				break;
-			case 24:
-				attr &= ~ATTR_UNDERSCORE;
-				break;
-			case 30:
-			case 31:
-			case 32:
-			case 33:
-			case 34:
-			case 35:
-			case 36:
-			case 37:
-				colr &= 0x0f;
-				colr |= (m - 30) << 4;
-				break;
-			case 39:
-				colr &= 0x0f;
-				colr |= 0x80;
-				break;
-			case 40:
-			case 41:
-			case 42:
-			case 43:
-			case 44:
-			case 45:
-			case 46:
-			case 47:
-				colr &= 0xf0;
-				colr |= m - 40;
-				break;
-			case 49:
-				colr &= 0xf0;
-				colr |= 0x08;
-				break;
+			if (o != 5)
+				continue;
+
+			i++;
+			if (input_get_argument(ictx, i, &o, 0) != 0)
+				return;
+			if (m == 38) {
+				gc->flags |= GRID_FLAG_FG256;
+				gc->fg = o;
+			} else if (m == 48) {
+				gc->flags |= GRID_FLAG_BG256;
+				gc->bg = o;
 			}
+			continue;
+		}
+
+		switch (m) {
+		case 0:
+		case 10:
+			attr = gc->attr;
+			memcpy(gc, &grid_default_cell, sizeof *gc);
+			gc->attr |= (attr & GRID_ATTR_CHARSET);
+			break;
+		case 1:
+			gc->attr |= GRID_ATTR_BRIGHT;
+			break;
+		case 2:
+			gc->attr |= GRID_ATTR_DIM;
+			break;
+		case 3:
+			gc->attr |= GRID_ATTR_ITALICS;
+			break;
+		case 4:
+			gc->attr |= GRID_ATTR_UNDERSCORE;
+			break;
+		case 5:
+			gc->attr |= GRID_ATTR_BLINK;
+			break;
+		case 7:
+			gc->attr |= GRID_ATTR_REVERSE;
+			break;
+		case 8:
+			gc->attr |= GRID_ATTR_HIDDEN;
+			break;
+		case 22:
+			gc->attr &= ~(GRID_ATTR_BRIGHT|GRID_ATTR_DIM);
+			break;
+		case 23:
+			gc->attr &= ~GRID_ATTR_ITALICS;
+			break;
+		case 24:
+			gc->attr &= ~GRID_ATTR_UNDERSCORE;
+			break;
+		case 25:
+			gc->attr &= ~GRID_ATTR_BLINK;
+			break;
+		case 27:
+			gc->attr &= ~GRID_ATTR_REVERSE;
+			break;
+		case 30:
+		case 31:
+		case 32:
+		case 33:
+		case 34:
+		case 35:
+		case 36:
+		case 37:
+			gc->flags &= ~GRID_FLAG_FG256;
+			gc->fg = m - 30;
+			break;
+		case 39:
+			gc->flags &= ~GRID_FLAG_FG256;
+			gc->fg = 8;
+			break;
+		case 40:
+		case 41:
+		case 42:
+		case 43:
+		case 44:
+		case 45:
+		case 46:
+		case 47:
+			gc->flags &= ~GRID_FLAG_BG256;
+			gc->bg = m - 40;
+			break;
+		case 49:
+			gc->flags &= ~GRID_FLAG_BG256;
+			gc->bg = 8;
+			break;
 		}
 	}
-	screen_write_set_attributes(&ictx->ctx, attr, colr);
 }
