@@ -1,4 +1,4 @@
-/* $Id: tmux.h,v 1.172 2008/06/30 05:34:06 nicm Exp $ */
+/* $Id: tmux.h,v 1.198 2008/11/17 18:56:36 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,10 +19,13 @@
 #ifndef TMUX_H
 #define TMUX_H
 
+#define PROTOCOL_VERSION -3
+
 /* Shut up gcc warnings about empty if bodies. */
 #define RB_AUGMENT(x) do {} while (0)
 
 #include <sys/param.h>
+#include <sys/time.h>
 
 #ifndef NO_QUEUE_H
 #include <sys/queue.h>
@@ -36,9 +39,15 @@
 #include "compat/tree.h"
 #endif
 
+#ifndef BROKEN_POLL
+#include <poll.h>
+#else
+#undef HAVE_POLL
+#include "compat/bsd-poll.h"
+#endif
+
 #include <ncurses.h>
 #include <limits.h>
-#include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdint.h>
@@ -49,6 +58,10 @@
 
 extern const char    *__progname;
 
+#ifndef GETOPT_PREFIX
+#define GETOPT_PREFIX ""
+#endif
+
 #ifndef INFTIM
 #define INFTIM -1
 #endif
@@ -68,21 +81,21 @@ extern const char    *__progname;
 #define __packed __attribute__ ((__packed__))
 #endif
 
-#ifndef timespeccmp
-#define timespeccmp(tsp, usp, cmp)                                      \
-        (((tsp)->tv_sec == (usp)->tv_sec) ?                             \
-            ((tsp)->tv_nsec cmp (usp)->tv_nsec) :                       \
-            ((tsp)->tv_sec cmp (usp)->tv_sec))
+#ifndef timercmp
+#define	timercmp(tvp, uvp, cmp)						\
+	(((tvp)->tv_sec == (uvp)->tv_sec) ?				\
+	    ((tvp)->tv_usec cmp (uvp)->tv_usec) :			\
+	    ((tvp)->tv_sec cmp (uvp)->tv_sec))
 #endif
 
-#ifndef timespecadd
-#define	timespecadd(tsp, usp, vsp)					\
+#ifndef timeradd
+#define	timeradd(tvp, uvp, vvp)						\
 	do {								\
-		(vsp)->tv_sec = (tsp)->tv_sec + (usp)->tv_sec;		\
-		(vsp)->tv_nsec = (tsp)->tv_nsec + (usp)->tv_nsec;	\
-		if ((vsp)->tv_nsec >= 1000000000L) {			\
-			(vsp)->tv_sec++;				\
-			(vsp)->tv_nsec -= 1000000000L;			\
+		(vvp)->tv_sec = (tvp)->tv_sec + (uvp)->tv_sec;		\
+		(vvp)->tv_usec = (tvp)->tv_usec + (uvp)->tv_usec;	\
+		if ((vvp)->tv_usec >= 1000000) {			\
+			(vvp)->tv_sec++;				\
+			(vvp)->tv_usec -= 1000000;			\
 		}							\
 	} while (0)
 #endif
@@ -113,17 +126,6 @@ extern const char    *__progname;
 #define printflike3 __attribute__ ((format (printf, 3, 4)))
 #define printflike4 __attribute__ ((format (printf, 4, 5)))
 
-/* Ensure buffer size. */
-#define ENSURE_SIZE(buf, len, size) do {				\
-	(buf) = ensure_size(buf, &(len), 1, size);			\
-} while (0)
-#define ENSURE_SIZE2(buf, len, nmemb, size) do {			\
-	(buf) = ensure_size(buf, &(len), nmemb, size);			\
-} while (0)
-#define ENSURE_FOR(buf, len, size, adj) do {				\
-	(buf) = ensure_for(buf, &(len), size, adj);			\
-} while (0)
-
 /* Buffer macros. */
 #define BUFFER_USED(b) ((b)->size)
 #define BUFFER_FREE(b) ((b)->space - (b)->off - (b)->size)
@@ -153,14 +155,14 @@ struct buffer {
 #define KEYC_REMOVEESCAPE(k) ((k) & ~KEYC_ESCAPE)
 #define KEYC_ISESCAPE(k) ((k) != KEYC_NONE && ((k) & KEYC_ESCAPE))
 
-#define KEYC_A1 (KEYC_OFFSET + 0x1)
-#define KEYC_A3 (KEYC_OFFSET + 0x2)
-#define KEYC_B2 (KEYC_OFFSET + 0x3)
+/* #define KEYC_A1 (KEYC_OFFSET + 0x1) */
+/* #define KEYC_A3 (KEYC_OFFSET + 0x2) */
+/* #define KEYC_B2 (KEYC_OFFSET + 0x3) */
 /* #define KEYC_BACKSPACE (KEYC_OFFSET + 0x4) */
 #define KEYC_BEG (KEYC_OFFSET + 0x5)
 #define KEYC_BTAB (KEYC_OFFSET + 0x6)
-#define KEYC_C1 (KEYC_OFFSET + 0x7)
-#define KEYC_C3 (KEYC_OFFSET + 0x8)
+/* #define KEYC_C1 (KEYC_OFFSET + 0x7) */
+/* #define KEYC_C3 (KEYC_OFFSET + 0x8) */
 #define KEYC_CANCEL (KEYC_OFFSET + 0x9)
 #define KEYC_CATAB (KEYC_OFFSET + 0xa)
 #define KEYC_CLEAR (KEYC_OFFSET + 0xb)
@@ -174,7 +176,7 @@ struct buffer {
 #define KEYC_DOWN (KEYC_OFFSET + 0x13)
 #define KEYC_EIC (KEYC_OFFSET + 0x14)
 #define KEYC_END (KEYC_OFFSET + 0x15)
-#define KEYC_ENTER (KEYC_OFFSET + 0x16)
+/* #define KEYC_ENTER (KEYC_OFFSET + 0x16) */
 #define KEYC_EOL (KEYC_OFFSET + 0x17)
 #define KEYC_EOS (KEYC_OFFSET + 0x18)
 #define KEYC_EXIT (KEYC_OFFSET + 0x19)
@@ -304,33 +306,51 @@ struct buffer {
 #define KEYC_UP (KEYC_OFFSET + 0x95)
 #define KEYC_MOUSE (KEYC_OFFSET + 0x96)
 
+/* Numeric keypad. Numbered from top-left, KPY_X. */
+#define KEYC_KP0_1 (KEYC_OFFSET + 0x100)
+#define KEYC_KP0_2 (KEYC_OFFSET + 0x101)
+#define KEYC_KP0_3 (KEYC_OFFSET + 0x102)
+#define KEYC_KP1_0 (KEYC_OFFSET + 0x103)
+#define KEYC_KP1_1 (KEYC_OFFSET + 0x104)
+#define KEYC_KP1_2 (KEYC_OFFSET + 0x105)
+#define KEYC_KP1_3 (KEYC_OFFSET + 0x106)
+#define KEYC_KP2_0 (KEYC_OFFSET + 0x107)
+#define KEYC_KP2_1 (KEYC_OFFSET + 0x108)
+#define KEYC_KP2_2 (KEYC_OFFSET + 0x109)
+#define KEYC_KP3_0 (KEYC_OFFSET + 0x10a)
+#define KEYC_KP3_1 (KEYC_OFFSET + 0x10b)
+#define KEYC_KP3_2 (KEYC_OFFSET + 0x10c)
+#define KEYC_KP3_3 (KEYC_OFFSET + 0x10d)
+#define KEYC_KP4_0 (KEYC_OFFSET + 0x10e)
+#define KEYC_KP4_2 (KEYC_OFFSET + 0x10f)
+
 /* Output codes. */
-#define TTY_CHARACTER 0
-#define TTY_CURSORUP 1
-#define TTY_CURSORDOWN 2
-#define TTY_CURSORRIGHT 3
-#define TTY_CURSORLEFT 4
-#define TTY_INSERTCHARACTER 5
-#define TTY_DELETECHARACTER 6
-#define TTY_INSERTLINE 7
-#define TTY_DELETELINE 8
-#define TTY_CLEARLINE 9
-#define TTY_CLEARENDOFLINE 10
-#define TTY_CLEARSTARTOFLINE 11
-#define TTY_CURSORMOVE 12
-#define TTY_ATTRIBUTES 13
-#define TTY_CURSOROFF 14
-#define TTY_CURSORON 15
-#define TTY_REVERSEINDEX 16
-#define TTY_SCROLLREGION 17
-#define TTY_INSERTON 18
-#define TTY_INSERTOFF 19
-#define TTY_KCURSOROFF 20
-#define TTY_KCURSORON 21
-#define TTY_KKEYPADOFF 22
-#define TTY_KKEYPADON 23
-#define TTY_MOUSEON 24
-#define TTY_MOUSEOFF 25 /* XXX merge allon/off into 1 arg? */
+#define TTY_CURSORUP 0
+#define TTY_CURSORDOWN 1
+#define TTY_CURSORRIGHT 2
+#define TTY_CURSORLEFT 3
+#define TTY_INSERTCHARACTER 4
+#define TTY_DELETECHARACTER 5
+#define TTY_INSERTLINE 6
+#define TTY_DELETELINE 7
+#define TTY_CLEARLINE 8
+#define TTY_CLEARENDOFLINE 9
+#define TTY_CLEARSTARTOFLINE 10
+#define TTY_CURSORMOVE 11
+#define TTY_CURSORMODE 12
+#define TTY_REVERSEINDEX 13
+#define TTY_SCROLLREGION 14
+#define TTY_INSERTMODE 15
+#define TTY_MOUSEMODE 16
+#define TTY_LINEFEED 17
+#define TTY_CARRIAGERETURN 18
+#define TTY_BELL 19
+#define TTY_KCURSORMODE 20
+#define TTY_KKEYPADMODE 21
+#define TTY_CLEARENDOFSCREEN 22
+#define TTY_CLEARSTARTOFSCREEN 23
+#define TTY_CLEARSCREEN 24
+#define TTY_CELL 25
 
 /* Message codes. */
 enum hdrtype {
@@ -353,7 +373,7 @@ struct hdr {
 };
 
 struct msg_command_data {
-	long long	pid;			/* pid from $TMUX or -1 */
+	pid_t		pid;			/* pid from $TMUX or -1 */
 	u_int		idx;			/* index from $TMUX */
 
 	size_t		namelen;
@@ -361,6 +381,12 @@ struct msg_command_data {
 
 struct msg_identify_data {
 	char		tty[TTY_NAME_MAX];
+	int	        version;
+
+#define IDENTIFY_UTF8 0x1
+#define IDENTIFY_256COLOURS 0x2
+#define IDENTIFY_HASDEFAULTS 0x4
+	int		flags;
 
 	u_int		sx;
 	u_int		sy;
@@ -373,23 +399,52 @@ struct msg_resize_data {
 	u_int		sy;
 };
 
-/* Attributes. */
-#define ATTR_BRIGHT 0x1
-#define ATTR_DIM 0x2
-#define ATTR_UNDERSCORE 0x4
-#define ATTR_BLINK 0x8
-#define ATTR_REVERSE 0x10
-#define ATTR_HIDDEN 0x20
-#define ATTR_ITALICS 0x40
-#define ATTR_CHARSET 0x80	/* alternative character set */
-
 /* Modes. */
-#define MODE_CURSOR 0x01
-#define MODE_INSERT 0x02
-#define MODE_KCURSOR 0x04
-#define MODE_KKEYPAD 0x08
-#define MODE_SAVED 0x10
-#define MODE_MOUSE 0x20
+#define MODE_CURSOR 0x1
+#define MODE_INSERT 0x2
+#define MODE_KCURSOR 0x4
+#define MODE_KKEYPAD 0x8
+#define MODE_MOUSE 0x10
+
+/* Grid output. */
+#define GRID_DEBUG(gd, fmt, ...) log_debug3("%s: (sx=%u, sy=%u, hsize=%u) " \
+    fmt, __func__, (gd)->sx, (gd)->sy, (gd)->hsize, ## __VA_ARGS__)
+
+/* Grid attributes. */
+#define GRID_ATTR_BRIGHT 0x1
+#define GRID_ATTR_DIM 0x2
+#define GRID_ATTR_UNDERSCORE 0x4
+#define GRID_ATTR_BLINK 0x8
+#define GRID_ATTR_REVERSE 0x10
+#define GRID_ATTR_HIDDEN 0x20
+#define GRID_ATTR_ITALICS 0x40
+#define GRID_ATTR_CHARSET 0x80	/* alternative character set */
+
+/* Grid flags. */
+#define GRID_FLAG_FG256 0x1
+#define GRID_FLAG_BG256 0x2
+#define GRID_FLAG_PADDING 0x4
+
+/* Grid cell. */
+struct grid_cell {
+	u_short	data;
+	u_char	attr;
+	u_char	flags;
+	u_char	fg;
+	u_char	bg;
+} __packed;
+
+/* Grid data. */
+struct grid_data {
+	u_int	sx;
+	u_int	sy;
+
+	u_int	hsize;
+	u_int	hlimit;
+
+	u_int  *size;
+	struct grid_cell **data;
+};
 
 /* Screen selection. */
 struct screen_sel {
@@ -406,29 +461,13 @@ struct screen_sel {
 struct screen {
 	char		*title;
 
-	u_char	       **grid_data;
-	u_char	       **grid_attr;
-	u_char	       **grid_colr;
-	u_int		*grid_size;
-
- 	u_int		 dx;		/* display x size */
-	u_int		 dy;		/* display y size */
-
-	u_int		 hsize;		/* history y size */
-	u_int		 hlimit;	/* history y limit */
-
-	u_int		 rupper;	/* scroll region top */
-	u_int		 rlower;	/* scroll region bottom */
+	struct grid_data *grid;		/* grid data */
 
 	u_int		 cx;		/* cursor x */
 	u_int		 cy;		/* cursor y */
-	u_char		 attr;
-	u_char		 colr;		/* fg:bg */
 
-	u_int		 saved_cx;
-	u_int		 saved_cy;
-	u_char		 saved_attr;
-	u_char		 saved_colr;
+	u_int		 rupper;	/* scroll region top */
+	u_int		 rlower;	/* scroll region bottom */
 
 	int		 mode;
 
@@ -454,56 +493,11 @@ struct screen_write_ctx {
 	struct screen	*s;
 };
 
-/* Screen display access macros. */
-#define screen_x(s, x) (x)
-#define screen_y(s, y) ((s)->hsize + y)
-
-#define screen_last_x(s) ((s)->dx - 1)
-#define screen_last_y(s) ((s)->dy - 1)
-
-#define screen_size_x(s) ((s)->dx)
-#define screen_size_y(s) ((s)->dy)
-
-#define screen_in_x(s, x) ((x) < screen_size_x(s))
-#define screen_in_y(s, y) ((y) < screen_size_y(s))
-#define screen_in_region(s, y) ((y) >= (s)->rupper && (y) <= (s)->rlower)
-
-/* These are inclusive... */
-#define screen_left_x(s, x) ((x) + 1)
-#define screen_right_x(s, x) \
-	((x) < screen_size_x(s) ? screen_size_x(s) - (x) : 0)
-
-#define screen_above_y(s, y) ((y) + 1)
-#define screen_below_y(s, y) \
-	((y) < screen_size_y(s) ? screen_size_y(s) - (y) : 0)
-
-#define SCREEN_DEBUG(s) do {						\
-	log_warnx("%s: cx=%u,cy=%u sx=%u,sy=%u", __func__,		\
-	    s->cx, s->cy, screen_size_x(s), screen_size_y(s));		\
-} while (0)
-#define SCREEN_DEBUG1(s, n) do {					\
-	log_warnx("%s: cx=%u,cy=%u sx=%u,sy=%u n=%u m=%u", __func__,	\
-	    s->cx, s->cy, screen_size_x(s), screen_size_y(s), n);	\
-} while (0)
-#define SCREEN_DEBUG2(s, n, m) do {					\
-	log_warnx("%s: cx=%u,cy=%u sx=%u,sy=%u n=%u m=%u", __func__,	\
-	    s->cx, s->cy, screen_size_x(s), screen_size_y(s), n, m);	\
-} while (0)
-#define SCREEN_DEBUG3(s, n, m, o) do {					\
-	log_warnx("%s: cx=%u,cy=%u sx=%u,sy=%u n=%u m=%u o=%u",		\
-	    __func__, s->cx, s->cy, screen_size_x(s), screen_size_y(s), \
-	    n, m, o);							\
-} while (0)
-#define SCREEN_DEBUG4(s, n, m, o, p) do {				\
-	log_warnx("%s: cx=%u,cy=%u sx=%u,sy=%u n=%u m=%u o=%u p=%u",	\
-	    __func__, s->cx, s->cy, screen_size_x(s), screen_size_y(s), \
-	    n, m, o, p);					       	\
-} while (0)
-
-/* Screen default contents. */
-#define SCREEN_DEFDATA ' '
-#define SCREEN_DEFATTR 0
-#define SCREEN_DEFCOLR 0x88	/* default on default ;-) */
+/* Screen size. */
+#define screen_size_x(s) ((s)->grid->sx)
+#define screen_size_y(s) ((s)->grid->sy)
+#define screen_hsize(s) ((s)->grid->hsize)
+#define screen_hlimit(s) ((s)->grid->hlimit)
 
 /* Input parser sequence argument. */
 struct input_arg {
@@ -520,14 +514,25 @@ struct input_ctx {
 	size_t		 len;
 	size_t		 off;
 
+	struct grid_cell cell;		/* current cell data */
+
+	struct grid_cell saved_cell;
+	u_int		 saved_cx;
+	u_int		 saved_cy;
+
 #define MAXSTRINGLEN	1024
 	u_char		*string_buf;
 	size_t		 string_len;
 	int		 string_type;
-#define STRING_TITLE 0
-#define STRING_NAME 1
-#define STRING_IGNORE 2
+#define STRING_SYSTEM 0
+#define STRING_APPLICATION 1
+#define STRING_NAME 2
 
+	u_char		 utf8_buf[4];
+	u_int		 utf8_len;
+	u_int		 utf8_off;
+
+	u_char		 intermediate;
 	void 		*(*state)(u_char, struct input_ctx *);
 
 	u_char		 private;
@@ -564,6 +569,7 @@ struct window {
 #define WINDOW_MONITOR 0x8
 #define WINDOW_AGGRESSIVE 0x10
 #define WINDOW_ZOMBIFY 0x20
+#define WINDOW_UTF8 0x40
 
 	u_int		 limitx;
 	u_int		 limity;
@@ -584,8 +590,10 @@ struct winlink {
 	struct window	*window;
 
 	RB_ENTRY(winlink) entry;
+	SLIST_ENTRY(winlink) sentry;
 };
 RB_HEAD(winlinks, winlink);
+SLIST_HEAD(winlink_stack, winlink);
 
 /* Option data structures. */
 struct options_entry {
@@ -615,7 +623,7 @@ struct options {
 /* Paste buffer. */
 struct paste_buffer {
      	char		*data;
-	struct timespec	 ts;
+	struct timeval	 tv;
 };
 ARRAY_DECL(paste_stack, struct paste_buffer *);
 
@@ -624,25 +632,25 @@ struct session_alert {
 	struct winlink	*wl;
 	int		 type;
 
-	TAILQ_ENTRY(session_alert) entry;
+	SLIST_ENTRY(session_alert) entry;
 };
 
 struct session {
 	char		*name;
-	struct timespec	 ts;
+	struct timeval	 tv;
 
 	u_int		 sx;
 	u_int		 sy;
 
 	struct winlink	*curw;
-	struct winlink	*lastw;
+	struct winlink_stack lastw;
 	struct winlinks	 windows;
 
 	struct options	 options;
 
 	struct paste_stack buffers;
 
-	TAILQ_HEAD(, session_alert) alerts;
+	SLIST_HEAD(, session_alert) alerts;
 
 #define SESSION_UNATTACHED 0x1	/* not attached to any clients */
 	int		 flags;
@@ -662,7 +670,11 @@ struct tty_term {
 	TERMINAL	*term;
 	u_int		 references;
 
-	TAILQ_ENTRY(tty_term) entry;
+#define TERM_HASDEFAULTS 0x1
+#define TERM_256COLOURS 0x2
+	int		 flags;
+
+	SLIST_ENTRY(tty_term) entry;
 };
 
 struct tty {
@@ -675,19 +687,23 @@ struct tty {
 	struct buffer	*in;
 	struct buffer	*out;
 
+	int		 log_fd;
+
 	struct termios   tio;
 
-	u_int		 attr;
-	u_int		 colr;
+	struct grid_cell cell;
 
 	u_char		 acs[UCHAR_MAX + 1];
 
 #define TTY_NOCURSOR 0x1
 #define TTY_FREEZE 0x2
 #define TTY_ESCAPE 0x4
+#define TTY_UTF8 0x8
 	int		 flags;
 
-	struct timespec	 key_timer;
+	int		 term_flags;
+
+	struct timeval	 key_timer;
 
 	size_t		 ksize;	/* maximum key size */
 	RB_HEAD(tty_keys, tty_key) ktree;
@@ -702,7 +718,7 @@ struct client {
 	char		*title;
 
 	struct tty 	 tty;
-	struct timespec	 status_timer;
+	struct timeval	 status_timer;
 
 	u_int		 sx;
 	u_int		 sy;
@@ -715,7 +731,7 @@ struct client {
 	int		 flags;
 
 	char		*message_string;
-	struct timespec	 message_timer;
+	struct timeval	 message_timer;
 
 	char		*prompt_string;
 	char		*prompt_buffer;
@@ -812,7 +828,7 @@ struct set_option_entry {
 	enum {
 		SET_OPTION_STRING,
 		SET_OPTION_NUMBER,
-		SET_OPTION_KEY,		
+		SET_OPTION_KEY,
 		SET_OPTION_COLOUR,
 		SET_OPTION_FLAG,
 		SET_OPTION_CHOICE
@@ -820,11 +836,32 @@ struct set_option_entry {
 
 	u_int		 minimum;
 	u_int		 maximum;
-	
+
 	const char     **choices;
 };
 extern const struct set_option_entry set_option_table[];
-#define NSETOPTION 14
+#define NSETOPTION 16
+
+/* Edit keys. */
+enum mode_key {
+	MODEKEY_BOL,
+	MODEKEY_CLEARSEL,
+	MODEKEY_COPYSEL,
+	MODEKEY_DOWN,
+	MODEKEY_EOL,
+	MODEKEY_LEFT,
+	MODEKEY_NONE,
+	MODEKEY_NPAGE,
+	MODEKEY_NWORD,
+	MODEKEY_PPAGE,
+	MODEKEY_PWORD,
+	MODEKEY_QUIT,
+	MODEKEY_RIGHT,
+	MODEKEY_STARTSEL,
+	MODEKEY_UP,
+};
+#define MODEKEY_EMACS 0
+#define MODEKEY_VI 1
 
 #ifdef NO_STRTONUM
 /* strtonum.c */
@@ -876,6 +913,9 @@ void		 sigreset(void);
 /* cfg.c */
 int		 load_cfg(const char *, char **x);
 
+/* mode-key.c */
+enum mode_key	 mode_key_lookup(int, int);
+
 /* options.c */
 int	options_cmp(struct options_entry *, struct options_entry *);
 SPLAY_PROTOTYPE(options_tree, options_entry, entry, options_cmp);
@@ -989,7 +1029,7 @@ extern const struct cmd_entry cmd_unbind_key_entry;
 extern const struct cmd_entry cmd_unlink_window_entry;
 
 /* cmd-string.c */
-struct cmd *cmd_string_parse(const char *, char **);
+int	cmd_string_parse(const char *, struct cmd **, char **);
 
 /* cmd-generic.c */
 #define CMD_TARGET_WINDOW_USAGE "[-t target-window]"
@@ -1024,7 +1064,7 @@ void	cmd_buffer_free(struct cmd *);
 void	cmd_buffer_print(struct cmd *, char *, size_t);
 
 /* client.c */
-int	 client_init(const char *, struct client_ctx *, int);
+int	 client_init(const char *, struct client_ctx *, int, int);
 int	 client_flush(struct client_ctx *);
 int	 client_main(struct client_ctx *);
 
@@ -1097,61 +1137,90 @@ void	 input_parse(struct window *);
 /* input-key.c */
 void	 input_key(struct window *, int);
 
-/* screen-display.c */
-void	 screen_display_set_cell(
-    	     struct screen *, u_int, u_int, u_char, u_char, u_char);
-void	 screen_display_make_lines(struct screen *, u_int, u_int);
-void	 screen_display_free_lines(struct screen *, u_int, u_int);
-void	 screen_display_move_lines(struct screen *, u_int, u_int, u_int);
-void	 screen_display_fill_area(struct screen *,
-    	     u_int, u_int, u_int, u_int, u_char, u_char, u_char);
-void	 screen_display_scroll_region_up(struct screen *);
-void	 screen_display_scroll_region_down(struct screen *);
-void	 screen_display_insert_lines(struct screen *, u_int, u_int);
-void	 screen_display_insert_lines_region(struct screen *, u_int, u_int);
-void	 screen_display_delete_lines(struct screen *, u_int, u_int);
-void	 screen_display_delete_lines_region(struct screen *, u_int, u_int);
-void	 screen_display_insert_characters(struct screen *, u_int, u_int, u_int);
-void	 screen_display_delete_characters(struct screen *, u_int, u_int, u_int);
-void	 screen_display_copy_area(struct screen *, struct screen *,
-    	     u_int, u_int, u_int, u_int, u_int, u_int);
+/* colour.c */
+const char *colour_tostring(u_char);
+u_char	 colour_fromstring(const char *);
+u_char	 colour_translate256(u_char);
+
+/* grid.c */
+extern const struct grid_cell grid_default_cell;
+struct grid_data *grid_create(u_int, u_int, u_int);
+void	 grid_destroy(struct grid_data *);
+void	 grid_reduce_line(struct grid_data *, u_int, u_int);
+void	 grid_expand_line(struct grid_data *, u_int, u_int);
+void	 grid_scroll_line(struct grid_data *);
+const struct grid_cell *grid_peek_cell(struct grid_data *, u_int, u_int);
+struct grid_cell *grid_get_cell(struct grid_data *, u_int, u_int);
+void	 grid_set_cell(
+	     struct grid_data *, u_int, u_int, const struct grid_cell *);
+void	 grid_clear(struct grid_data *, u_int, u_int, u_int, u_int);
+void	 grid_fill(struct grid_data *,
+    	     const struct grid_cell *, u_int, u_int, u_int, u_int);
+void	 grid_fill_lines(
+    	     struct grid_data *, const struct grid_cell *, u_int, u_int);
+void	 grid_clear_lines(struct grid_data *, u_int, u_int);
+void	 grid_move_lines(struct grid_data *, u_int, u_int, u_int);
+void	 grid_clear_cells(struct grid_data *, u_int, u_int, u_int);
+void	 grid_move_cells(struct grid_data *, u_int, u_int, u_int, u_int);
+
+/* grid-view.c */
+const struct grid_cell *grid_view_peek_cell(struct grid_data *, u_int, u_int);
+struct grid_cell *grid_view_get_cell(struct grid_data *, u_int, u_int);
+void	 grid_view_set_cell(
+	     struct grid_data *, u_int, u_int, const struct grid_cell *);
+void	 grid_view_clear(struct grid_data *, u_int, u_int, u_int, u_int);
+void	 grid_view_fill(struct grid_data *,
+    	     const struct grid_cell *, u_int, u_int, u_int, u_int);
+void	 grid_view_scroll_region_up(struct grid_data *, u_int, u_int);
+void	 grid_view_scroll_region_down(struct grid_data *, u_int, u_int);
+void	 grid_view_insert_lines(struct grid_data *, u_int, u_int);
+void	 grid_view_insert_lines_region(
+    	     struct grid_data *, u_int, u_int, u_int, u_int);
+void	 grid_view_delete_lines(struct grid_data *, u_int, u_int);
+void	 grid_view_delete_lines_region(
+	     struct grid_data *, u_int, u_int, u_int, u_int);
+void	 grid_view_insert_cells(struct grid_data *, u_int, u_int, u_int);
+void	 grid_view_delete_cells(struct grid_data *, u_int, u_int, u_int);
 
 /* screen-write.c */
 void	 screen_write_start_window(struct screen_write_ctx *, struct window *);
 void	 screen_write_start_client(struct screen_write_ctx *, struct client *);
 void	 screen_write_start_session(
-    	    struct screen_write_ctx *, struct session *);
+    	     struct screen_write_ctx *, struct session *);
 void	 screen_write_start(struct screen_write_ctx *,
-    	    struct screen *, void (*)(void *, int, ...), void *);
+	     struct screen *, void (*)(void *, int, ...), void *);
 void	 screen_write_stop(struct screen_write_ctx *);
-void	 screen_write_set_title(struct screen_write_ctx *, char *);
-void	 screen_write_put_character(struct screen_write_ctx *, u_char);
-size_t printflike2 screen_write_put_string_rjust(
-	     struct screen_write_ctx *, const char *, ...);
-void printflike2 screen_write_put_string(
-	     struct screen_write_ctx *, const char *, ...);
-void	 screen_write_set_attributes(struct screen_write_ctx *, u_char, u_char);
-void	 screen_write_set_region(struct screen_write_ctx *, u_int, u_int);
-void	 screen_write_cursor_up_scroll(struct screen_write_ctx *);
-void	 screen_write_cursor_down_scroll(struct screen_write_ctx *);
-void	 screen_write_cursor_up(struct screen_write_ctx *, u_int);
-void	 screen_write_cursor_down(struct screen_write_ctx *, u_int);
-void	 screen_write_cursor_left(struct screen_write_ctx *, u_int);
-void	 screen_write_cursor_right(struct screen_write_ctx *, u_int);
-void	 screen_write_delete_lines(struct screen_write_ctx *, u_int);
-void	 screen_write_delete_characters(struct screen_write_ctx *, u_int);
-void	 screen_write_insert_lines(struct screen_write_ctx *, u_int);
-void	 screen_write_insert_characters(struct screen_write_ctx *, u_int);
-void	 screen_write_move_cursor(struct screen_write_ctx *, u_int, u_int);
-void	 screen_write_fill_end_of_screen(struct screen_write_ctx *);
-void	 screen_write_fill_screen(struct screen_write_ctx *);
-void	 screen_write_fill_end_of_line(struct screen_write_ctx *);
-void	 screen_write_fill_start_of_line(struct screen_write_ctx *);
-void	 screen_write_fill_line(struct screen_write_ctx *);
-void	 screen_write_set_mode(struct screen_write_ctx *, int);
-void	 screen_write_clear_mode(struct screen_write_ctx *, int);
-void	 screen_write_copy_area(struct screen_write_ctx *,
-    	     struct screen *, u_int, u_int, u_int, u_int);
+void printflike3 screen_write_puts(
+	     struct screen_write_ctx *, struct grid_cell *, const char *, ...);
+void	 screen_write_putc(
+    	     struct screen_write_ctx *, struct grid_cell *, u_char);
+void	 screen_write_copy(struct screen_write_ctx *,
+	     struct screen *, u_int, u_int, u_int, u_int);
+void	 screen_write_cursorup(struct screen_write_ctx *, u_int);
+void	 screen_write_cursordown(struct screen_write_ctx *, u_int);
+void	 screen_write_cursorright(struct screen_write_ctx *, u_int);
+void	 screen_write_cursorleft(struct screen_write_ctx *, u_int);
+void	 screen_write_insertcharacter(struct screen_write_ctx *, u_int);
+void	 screen_write_deletecharacter(struct screen_write_ctx *, u_int);
+void	 screen_write_insertline(struct screen_write_ctx *, u_int);
+void	 screen_write_deleteline(struct screen_write_ctx *, u_int);
+void	 screen_write_clearline(struct screen_write_ctx *);
+void	 screen_write_clearendofline(struct screen_write_ctx *);
+void	 screen_write_clearstartofline(struct screen_write_ctx *);
+void	 screen_write_cursormove(struct screen_write_ctx *, u_int, u_int);
+void	 screen_write_cursormode(struct screen_write_ctx *, int);
+void	 screen_write_reverseindex(struct screen_write_ctx *);
+void	 screen_write_scrollregion(struct screen_write_ctx *, u_int, u_int);
+void	 screen_write_insertmode(struct screen_write_ctx *, int);
+void	 screen_write_mousemode(struct screen_write_ctx *, int);
+void	 screen_write_linefeed(struct screen_write_ctx *);
+void	 screen_write_carriagereturn(struct screen_write_ctx *);
+void	 screen_write_kcursormode(struct screen_write_ctx *, int);
+void	 screen_write_kkeypadmode(struct screen_write_ctx *, int);
+void	 screen_write_clearendofscreen(struct screen_write_ctx *);
+void	 screen_write_clearstartofscreen(struct screen_write_ctx *);
+void	 screen_write_clearscreen(struct screen_write_ctx *);
+void	 screen_write_cell(struct screen_write_ctx *, const struct grid_cell *);
 
 /* screen-redraw.c */
 void	screen_redraw_start_window(struct screen_redraw_ctx *, struct window *);
@@ -1161,36 +1230,25 @@ void	screen_redraw_start_session(
 void	screen_redraw_start(struct screen_redraw_ctx *,
     	    struct screen *, void (*)(void *, int, ...), void *);
 void	screen_redraw_stop(struct screen_redraw_ctx *);
-void	screen_redraw_move_cursor(struct screen_redraw_ctx *, u_int, u_int);
-void	screen_redraw_set_attributes(struct screen_redraw_ctx *, u_int, u_int);
-void printflike2 screen_redraw_write_string(
-    	     struct screen_redraw_ctx *, const char *, ...);
+void printflike3 screen_redraw_puts(
+     	    struct screen_redraw_ctx *, struct grid_cell *, const char *, ...);
+void	screen_redraw_putc(
+    	    struct screen_redraw_ctx *, struct grid_cell *, u_char);
 void	screen_redraw_cell(struct screen_redraw_ctx *, u_int, u_int);
-void	screen_redraw_area(
-    	    struct screen_redraw_ctx *, u_int, u_int, u_int, u_int);
 void	screen_redraw_lines(struct screen_redraw_ctx *, u_int, u_int);
 void	screen_redraw_columns(struct screen_redraw_ctx *, u_int, u_int);
 
 /* screen.c */
-const char *screen_colourstring(u_char);
-u_char	 screen_stringcolour(const char *);
-void	 screen_create(struct screen *, u_int, u_int, u_int);
-void	 screen_reset(struct screen *);
-void	 screen_destroy(struct screen *);
+void	 screen_init(struct screen *, u_int, u_int, u_int);
+void	 screen_reinit(struct screen *);
+void	 screen_free(struct screen *);
+void	 screen_set_title(struct screen *, const char *);
 void	 screen_resize(struct screen *, u_int, u_int);
-void	 screen_expand_line(struct screen *, u_int, u_int);
-void	 screen_reduce_line(struct screen *, u_int, u_int);
-void	 screen_get_cell(
-    	     struct screen *, u_int, u_int, u_char *, u_char *, u_char *);
-void	 screen_set_cell(struct screen *, u_int, u_int, u_char, u_char, u_char);
-void	 screen_make_lines(struct screen *, u_int, u_int);
-void	 screen_free_lines(struct screen *, u_int, u_int);
-void	 screen_move_lines(struct screen *, u_int, u_int, u_int);
-void	 screen_fill_area(struct screen *,
-    	     u_int, u_int, u_int, u_int, u_char, u_char, u_char);
 void	 screen_set_selection(struct screen *, u_int, u_int, u_int, u_int);
 void	 screen_clear_selection(struct screen *);
 int	 screen_check_selection(struct screen *, u_int, u_int);
+void	 screen_display_copy_area(struct screen *, struct screen *,
+    	     u_int, u_int, u_int, u_int, u_int, u_int);
 
 /* window.c */
 extern struct windows windows;
@@ -1205,8 +1263,10 @@ struct winlink	*winlink_add(struct winlinks *, struct window *, int);
 void		 winlink_remove(struct winlinks *, struct winlink *);
 struct winlink	*winlink_next(struct winlinks *, struct winlink *);
 struct winlink	*winlink_previous(struct winlinks *, struct winlink *);
+void		 winlink_stack_push(struct winlink_stack *, struct winlink *);
+void		 winlink_stack_remove(struct winlink_stack *, struct winlink *);
 struct window	*window_create(const char *,
-    const char *, const char **, u_int, u_int, u_int);
+		     const char *, const char **, u_int, u_int, u_int);
 int		 window_spawn(struct window *, const char *, const char **);
 void		 window_destroy(struct window *);
 int		 window_resize(struct window *, u_int, u_int);
@@ -1245,6 +1305,11 @@ int		 session_previous(struct session *);
 int		 session_select(struct session *, int);
 int		 session_last(struct session *);
 
+/* utf8.c */
+u_int	utf8_combine(const u_char [4]);
+void	utf8_split(u_int, u_char [4]);
+int	utf8_width(u_int);
+
 /* buffer.c */
 struct buffer 	*buffer_create(size_t);
 void		 buffer_destroy(struct buffer *);
@@ -1270,10 +1335,13 @@ int		 buffer_poll(struct pollfd *, struct buffer *, struct buffer *);
 void		 buffer_flush(int, struct buffer *n, struct buffer *);
 
 /* log.c */
-void		 log_open(FILE *, int, int);
+#define LOG_FACILITY LOG_DAEMON
+void		 log_open_syslog(int);
+void		 log_open_tty(int);
+void		 log_open_file(int, const char *);
 void		 log_close(void);
-void		 log_vwrite(FILE *, int, const char *, va_list);
-void		 log_write(FILE *, int, const char *, ...);
+void		 log_vwrite(int, const char *, va_list);
+void		 log_write(int, const char *, ...);
 void printflike1 log_warn(const char *, ...);
 void printflike1 log_warnx(const char *, ...);
 void printflike1 log_info(const char *, ...);
@@ -1285,9 +1353,6 @@ __dead void	 log_fatal(const char *, ...);
 __dead void	 log_fatalx(const char *, ...);
 
 /* xmalloc.c */
-void		*ensure_size(void *, size_t *, size_t, size_t);
-void		*ensure_for(void *, size_t *, size_t, size_t);
-char		*xmemstrdup(const char *, size_t);
 char		*xstrdup(const char *);
 void		*xcalloc(size_t, size_t);
 void		*xmalloc(size_t);
