@@ -1,4 +1,4 @@
-/* $Id: window-more.c,v 1.14 2008/06/22 16:56:47 nicm Exp $ */
+/* $Id: window-more.c,v 1.20 2008/09/26 06:45:28 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -62,7 +62,7 @@ window_more_vadd(struct window *w, const char *fmt, va_list ap)
 
 	screen_write_start_window(&ctx, w);
 	size = ARRAY_LENGTH(&data->list) - 1;
-	if (size >= data->top && size <= data->top + screen_last_y(s)) {
+	if (size >= data->top && size <= data->top + screen_size_y(s) - 1) {
 		window_more_write_line(w, &ctx, size - data->top);
 		if (size != data->top)
 			window_more_write_line(w, &ctx, 0);
@@ -92,7 +92,7 @@ window_more_init(struct window *w)
 	data->top = 0;
 
 	s = &data->screen;
-	screen_create(s, screen_size_x(&w->base), screen_size_y(&w->base), 0);
+	screen_init(s, screen_size_x(&w->base), screen_size_y(&w->base), 0);
 	s->mode &= ~MODE_CURSOR;
 
 	return (s);
@@ -108,7 +108,7 @@ window_more_free(struct window *w)
 		xfree(ARRAY_ITEM(&data->list, i));
 	ARRAY_FREE(&data->list);
 
-	screen_destroy(&data->screen);
+	screen_free(&data->screen);
 	xfree(data);
 }
 
@@ -123,42 +123,39 @@ window_more_resize(struct window *w, u_int sx, u_int sy)
 }
 
 void
-window_more_key(struct window *w, unused struct client *c, int key)
+window_more_key(struct window *w, struct client *c, int key)
 {
 	struct window_more_mode_data	*data = w->modedata;
 	struct screen			*s = &data->screen;
+	int				 table;
 
-	switch (key) {
-	case 'Q':
-	case 'q':
+	table = options_get_number(&c->session->options, "mode-keys");
+	switch (mode_key_lookup(table, key)) {
+	case MODEKEY_QUIT:
 		window_reset_mode(w);
 		break;
-	case 'k':
-	case 'K':
-	case KEYC_UP:
+	case MODEKEY_UP:
 		window_more_scroll_up(w);
 		break;
-	case 'j':
-	case 'J':
-	case KEYC_DOWN:
+	case MODEKEY_DOWN:
 		window_more_scroll_down(w);
 		break;
-	case '\025':	/* C-u */
-	case KEYC_PPAGE:
+	case MODEKEY_PPAGE:
 		if (data->top < screen_size_y(s))
 			data->top = 0;
 		else
 			data->top -= screen_size_y(s);
 		window_more_redraw_screen(w);
 		break;
-	case '\006':	/* C-f */
-	case KEYC_NPAGE:
+	case MODEKEY_NPAGE:
 		if (data->top + screen_size_y(s) > ARRAY_LENGTH(&data->list))
 			data->top = ARRAY_LENGTH(&data->list);
 		else
 			data->top += screen_size_y(s);
 		window_more_redraw_screen(w);
 		break;
+	default:
+		break;
 	}
 }
 
@@ -167,27 +164,30 @@ window_more_write_line(struct window *w, struct screen_write_ctx *ctx, u_int py)
 {
 	struct window_more_mode_data	*data = w->modedata;
 	struct screen			*s = &data->screen;
-	char   				*msg;
+	struct grid_cell		 gc;
+	char   				*msg, hdr[32];
 	size_t	 			 size;
 
+	memcpy(&gc, &grid_default_cell, sizeof gc);
+
 	if (py == 0) {
-		screen_write_set_attributes(
-		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x88);
-		screen_write_move_cursor(ctx, 0, 0);
-		size = screen_write_put_string_rjust(
-		    ctx, "[%u/%u]", data->top, ARRAY_LENGTH(&data->list));
+		size = xsnprintf(hdr, sizeof hdr,
+		    "[%u/%u]", data->top, ARRAY_LENGTH(&data->list));
+		screen_write_cursormove(ctx, screen_size_x(s) - size, 0);
+		gc.attr |= GRID_ATTR_BRIGHT|GRID_ATTR_REVERSE;
+		screen_write_puts(ctx, &gc, "%s", hdr);
+		gc.attr &= ~(GRID_ATTR_BRIGHT|GRID_ATTR_REVERSE);
 	} else
 		size = 0;
 
-	screen_write_set_attributes(ctx, SCREEN_DEFATTR, SCREEN_DEFCOLR);
-	screen_write_move_cursor(ctx, 0, py);
+	screen_write_cursormove(ctx, 0, py);
 	if (data->top + py  < ARRAY_LENGTH(&data->list)) {
 		msg = ARRAY_ITEM(&data->list, data->top + py);
-		screen_write_put_string(
-		    ctx, "%.*s", (int) (screen_size_x(s) - size), msg);
+		screen_write_puts(
+		    ctx, &gc, "%.*s", (int) (screen_size_x(s) - size), msg);
 	}
 	while (s->cx < screen_size_x(s) - size)
-		screen_write_put_character(ctx, SCREEN_DEFDATA);
+		screen_write_putc(ctx, &gc, ' ');
 }
 
 void
@@ -215,8 +215,8 @@ window_more_scroll_up(struct window *w)
 	data->top--;
 
 	screen_write_start_window(&ctx, w);
-	screen_write_move_cursor(&ctx, 0, 0);
-	screen_write_insert_lines(&ctx, 1);
+	screen_write_cursormove(&ctx, 0, 0);
+	screen_write_insertline(&ctx, 1);
 	window_more_write_line(w, &ctx, 0);
 	window_more_write_line(w, &ctx, 1);
 	screen_write_stop(&ctx);
@@ -234,9 +234,9 @@ window_more_scroll_down(struct window *w)
 	data->top++;
 
 	screen_write_start_window(&ctx, w);
-	screen_write_move_cursor(&ctx, 0, 0);
-	screen_write_delete_lines(&ctx, 1);
-	window_more_write_line(w, &ctx, screen_last_y(s));
+	screen_write_cursormove(&ctx, 0, 0);
+	screen_write_deleteline(&ctx, 1);
+	window_more_write_line(w, &ctx, screen_size_y(s) - 1);
 	window_more_write_line(w, &ctx, 0);
 	screen_write_stop(&ctx);
 }
