@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.47 2008/06/30 18:27:14 nicm Exp $ */
+/* $Id: window.c,v 1.51 2008/11/16 10:10:26 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -164,6 +164,32 @@ winlink_previous(unused struct winlinks *wwl, struct winlink *wl)
 #endif
 }
 
+void
+winlink_stack_push(struct winlink_stack *stack, struct winlink *wl)
+{
+	if (wl == NULL)
+		return;
+
+	winlink_stack_remove(stack, wl);
+	SLIST_INSERT_HEAD(stack, wl, sentry);
+}
+
+void
+winlink_stack_remove(struct winlink_stack *stack, struct winlink *wl)
+{
+	struct winlink	*wl2;
+
+	if (wl == NULL)
+		return;
+
+	SLIST_FOREACH(wl2, stack, sentry) {
+		if (wl2 == wl) {
+			SLIST_REMOVE(stack, wl, winlink, sentry);
+			return;
+		}
+	}
+}
+
 struct window *
 window_create(const char *name,
     const char *cmd, const char **envp, u_int sx, u_int sy, u_int hlimit)
@@ -182,7 +208,7 @@ window_create(const char *name,
 	w->flags = 0;
 
 	w->limitx = w->limity = UINT_MAX;
-	screen_create(&w->base, sx, sy, hlimit);
+	screen_init(&w->base, sx, sy, hlimit);
 	w->screen = &w->base;
 
 	input_init(w);
@@ -248,7 +274,7 @@ window_spawn(struct window *w, const char *cmd, const char **envp)
 				fatal("putenv failed");
 		}
 		sigreset();
-		log_debug("new child: cmd=%s; pid=%d", w->cmd, (int) getpid());
+		log_debug("new child: cmd=%s; pid=%ld", w->cmd, (long) getpid());
 		log_close();
 
 		execl(_PATH_BSHELL, "sh", "-c", w->cmd, (char *) NULL);
@@ -261,7 +287,7 @@ window_spawn(struct window *w, const char *cmd, const char **envp)
 		fatal("fcntl failed");
 	if (fcntl(w->fd, F_SETFD, FD_CLOEXEC) == -1)
 		fatal("fcntl failed");
-	
+
 	return (0);
 }
 
@@ -282,7 +308,7 @@ window_destroy(struct window *w)
 	input_free(w);
 
 	window_reset_mode(w);
-	screen_destroy(&w->base);
+	screen_free(&w->base);
 
 	buffer_destroy(w->in);
 	buffer_destroy(w->out);
